package net.wanji.utc.hisense.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.dao.entity.CrossSchedulesPO;
import net.wanji.databus.dao.mapper.CrossInfoMapper;
import net.wanji.databus.dao.mapper.ManufacturerInfoMapper;
import net.wanji.databus.dto.*;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.databus.po.CrossInfoPOExt;
import net.wanji.databus.vo.PlanSectionVO;
import net.wanji.databus.vo.SchemePhaseLightsVO;
import net.wanji.utc.hisense.common.OperationBaseDom;
import net.wanji.utc.hisense.common.constants.HttpConstants;
import net.wanji.utc.hisense.mapper.VIntersectionMapper;
import net.wanji.utc.hisense.mapper.VNtcipTimeBaseScheduleMapper;
import net.wanji.utc.hisense.pojo.result.CoordinationStatus;
import net.wanji.utc.hisense.pojo.result.CrossSchemeRings;
import net.wanji.utc.hisense.pojo.view.VIntersectionEntity;
import net.wanji.utc.hisense.pojo.view.VNtcipTimeBaseSchedule;
import net.wanji.utc.hisense.service.StaticInfoService;
import net.wanji.utc.hisense.util.OkHttpClientUtil;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.Node;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/05/08 17:29
 */
@Service
@RequiredArgsConstructor
@Slf4j
public class StaticInfoServiceImpl implements StaticInfoService {

    @Value("${hisense.url}")
    private String hisenseUrl;
    @Resource
    private CrossInfoMapper crossInfoMapper;
    @Resource
    private ManufacturerInfoMapper manufacturerInfoMapper;
    @Resource
    private VIntersectionMapper vIntersectionMapper;
    @Resource
    private VNtcipTimeBaseScheduleMapper vNtcipTimeBaseScheduleMapper;

    @Override
    public List<CrossInfoPO> crossBasicInfo(CrossInfoDTO crossInfoDTO) throws Exception {
        Integer id = manufacturerInfoMapper.selectIdByCode(crossInfoDTO.getManufacturerCode());
        List<CrossInfoPO> crossInfoPOList = crossInfoMapper.selectByManufacturerId(id);
        List<String> codeList = crossInfoPOList.stream().map(CrossInfoPO::getCode).collect(Collectors.toList());
        Map<String, String> codeCrossIdMap = crossInfoPOList.stream().collect(Collectors.toMap(CrossInfoPO::getCode, CrossInfoPO::getId));
        List<CrossInfoPO> crossInfoPOS = new ArrayList<>();
        if (StringUtils.equalsIgnoreCase("HISENSE", crossInfoDTO.getManufacturerCode())) {
            LambdaQueryWrapper<VIntersectionEntity> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.in(VIntersectionEntity::getCIntsID, codeList);
            List<VIntersectionEntity> vIntersectionEntities = vIntersectionMapper.selectList(queryWrapper);
            for (VIntersectionEntity vIntersectionEntity : vIntersectionEntities) {
                String cIntsID = vIntersectionEntity.getCIntsID();
                CrossInfoPO crossInfoPO = new CrossInfoPO();
                crossInfoPO.setId(codeCrossIdMap.get(cIntsID));
                crossInfoPO.setCode(cIntsID);
                crossInfoPO.setName(vIntersectionEntity.getCIntsName());
                crossInfoPO.setIp(vIntersectionEntity.getCCONTROLLERIP());
                String [] array = {String.valueOf(vIntersectionEntity.getLongitude()), String.valueOf(vIntersectionEntity.getLatitude())};
                crossInfoPO.setLocation(StringUtils.join(array, ","));
                crossInfoPO.setVersion(vIntersectionEntity.getCControllerModel());
                crossInfoPO.setManufacturerId(22);
                crossInfoPOS.add(crossInfoPO);
            }
        }
        return crossInfoPOS;
    }

    @Override
    public SchemePhaseLightsVO schemePhaseLights(SchemePhaseLightsDTO schemePhaseLightsDTO) throws Exception {
        return null;
    }

    @Override
    public List<PlanSectionVO> planSection(PlanSectionDTO planSectionDTO) throws Exception {
        return null;
    }

    @Override
    public List<CrossSchedulesPO> crossSchedules(CrossSchedulesDTO crossSchedulesDTO) throws Exception {

        String manufacturerCode = crossSchedulesDTO.getManufacturerCode();
        List<String> crossIdList = crossSchedulesDTO.getCrossIdList();
        if (!CollectionUtils.isEmpty(crossIdList)) {
            List<CrossSchedulesPO> result = new ArrayList<>(crossIdList.size());
            for (String crossId : crossIdList) {
                LambdaQueryWrapper<VNtcipTimeBaseSchedule> queryWrapper = new LambdaQueryWrapper<>();
                queryWrapper.eq(VNtcipTimeBaseSchedule::getCIntsID, crossId);
                List<VNtcipTimeBaseSchedule> vNtcipTimeBaseSchedules = vNtcipTimeBaseScheduleMapper.selectList(queryWrapper);
                if (!CollectionUtils.isEmpty(vNtcipTimeBaseSchedules)) {
                    for (VNtcipTimeBaseSchedule vNtcipTimeBaseSchedule : vNtcipTimeBaseSchedules) {
                        if (Objects.nonNull(vNtcipTimeBaseSchedule)) {
                            CrossSchedulesPO crossSchedulesPO = new CrossSchedulesPO();
                            crossSchedulesPO.setCrossId(crossId);
                            crossSchedulesPO.setScheduleNo(vNtcipTimeBaseSchedule.getNTimeBaseScheduleNumber());
                            crossSchedulesPO.setName("海信调度" + vNtcipTimeBaseSchedule.getNTimeBaseScheduleNumber());
                            if (8190 == vNtcipTimeBaseSchedule.getNTimeBaseScheduleMonth()) {
                                Integer nTimeBaseScheduleDay = vNtcipTimeBaseSchedule.getNTimeBaseScheduleDay();
                                //crossSchedulesPO.setWeek();
                            }


                        }
                    }
                }
            }
        }
        return null;
    }

    @Override
    public CoordinationStatus crossPlan(String crossId) throws Exception {
        CrossInfoPOExt crossInfoPOExt = crossInfoMapper.selectById(crossId);
        //组装document
        Document document = OperationBaseDom.generateBaseDoc();
        Element element = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION);
        element.addElement(HttpConstants.MESSAGETYPE).setText(HttpConstants.MESSAGETYPE_12);
        Element messageContent = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT);
        messageContent.addElement(HttpConstants.SPOT).setText(crossInfoPOExt.getCode());

        //给海信发送http请求
        String post = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
        if (StringUtils.isBlank(post)){
            log.error("请求 5.7. 协调状态 失败");
            return null;
        }
        Document postResult = DocumentHelper.parseText(post);
        String result = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
        if (HttpConstants.RESULT_1.equals(result)){
            //成功
            String spot = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("Spot");
            String CoordPatternStatus = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("CoordPatternStatus");
            String CoordCycleStatus = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("CoordCycleStatus");
            String CoordSyncStatus = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("CoordSyncStatus");
            CoordinationStatus coordinationStatus = CoordinationStatus.builder()
                    .spot(crossId)
                    .coordPatternStatus(CoordPatternStatus)
                    .coordCycleStatus(CoordCycleStatus)
                    .coordSyncStatus(CoordSyncStatus)
                    .build();
            return coordinationStatus;

        }else {
            //失败
            log.error("请求 5.7. 协调状态 失败");
        }

        return null;
    }

    @Override
    public CrossSchemeRings crossSchemeRings(CrossSchemeRingsDTO crossSchemeRingsDTO) throws Exception {
        CrossInfoPOExt crossInfoPOExt = crossInfoMapper.selectById(crossSchemeRingsDTO.getCrossId());
        //组装document
        Document document = OperationBaseDom.generateBaseDoc();
        Element element = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION);
        element.addElement(HttpConstants.MESSAGETYPE).setText(HttpConstants.MESSAGETYPE_30);
        Element messageContent = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT);
        messageContent.addElement(HttpConstants.SPOT).setText(crossInfoPOExt.getCode());
        messageContent.addElement(HttpConstants.PATTERN).setText(crossSchemeRingsDTO.getPattern());
        //给海信发送http请求
        String post = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
        if (StringUtils.isBlank(post)){
            log.error("请求 5.15.根据方案号获取配时方案环图 失败");
            return null;
        }
        Document postResult = DocumentHelper.parseText(post);
        String result = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
        if (HttpConstants.RESULT_1.equals(result)){
            //成功
            String patternRest = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("Pattern");
            String cycleLen = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("CycleLen");
            String ringCount = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("RingCount");
            String offset = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("Offset");
            List<Element> cycleElementList = postResult.selectNodes(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT + "/CycleList");
            List<Element> phaseElementList = postResult.selectNodes(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT + "/PhaseList");
            CrossSchemeRings crossSchemeRings = new CrossSchemeRings();
            crossSchemeRings.setSpot(crossSchemeRingsDTO.getCrossId());
            crossSchemeRings.setPattern(patternRest);
            crossSchemeRings.setRingCount(ringCount);
            crossSchemeRings.setCycleLen(cycleLen);
            crossSchemeRings.setOffset(offset);

            JSONObject jsonObject = new JSONObject();
            for (Element cycle : cycleElementList) {
                List<Node> elements = cycle.elements();
                for (Node node : elements) {
                    node.getName();
                    jsonObject.put(node.getName(), node.getText());
                }
            }
            List<CrossSchemeRings.Phase> phaseList = new ArrayList<>();
            for (Element el : phaseElementList) {
                CrossSchemeRings.Phase phase = new CrossSchemeRings.Phase();
                phase.setRedTime(el.valueOf("RedTime"));
                phase.setYellowTime(el.valueOf("YellowTime"));
                phase.setGreenTime(el.valueOf("GreenTime"));
                phase.setDirection(el.valueOf("Direction"));
                phase.setMinGTime(el.valueOf("MinGTime"));
                phase.setMaxGTime(el.valueOf("MaxGTime"));
                phase.setChannelDim(el.valueOf("ChannelDim"));
                phase.setConcurrent(el.valueOf("Concurrent"));
                phaseList.add(phase);
            }
            crossSchemeRings.setCycleList(jsonObject);
            crossSchemeRings.setPhaseList(phaseList);
            return crossSchemeRings;

        }else {
            //失败
            log.error("请求 5.15. 方案环图请求 失败");
        }
        return null;
    }
}
